import subprocess, random, string, os, fcntl

class LineBuffer(object):
	"""
	Processes each line in the desired buffer

	From Juan Luis Boya: https://github.com/ntrrgc/dotfiles/blob/master/wtfd
	"""
	def __init__(self):
		self.buffer = b''

	def read_lines(self, input):
		"""
		Processes each line and appends it to the buffer
		"""
		while b'\n' in input:
			before, after = input.split(b'\n', 1)
			yield self.buffer + before
			self.buffer = b''
			input = after
		self.buffer += input

class ProcessReactor(object):
	def __init__(self, user, directory, ioloop, ip, opensockets, *args, **kwargs):
		"""
		Starts the command and sets the redirection of the desired buffers
		:param pwd: user The pwd structure with the information of the user which issued the command
		:param str directory: The directory to use as cwd
		:param list: args A list of supplementary arguments
		:param dict: kwargs A dictionary of keyword arguments

		The function redirects STDOUT and STDERR to a pipe, and then executes the command using Popen.
		The output pipe descriptor is made non-blocking and included in the instance of the IOLoop. 
		"""

		self.user = user #user which executes the command
		self.command = ' '.join(*args) # The name of the command
		self.ip = ip # The IP of the server
		self.shell =  kwargs.get("shell", False)
		self.ioloop = ioloop
		def randomString():
			"""
			Generates a random token

			:returns: A random string which acts as a token
			"""
			return ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(3))

		# Generates a random token to identify the execution
		self.identifier = ''.join(random.choice(string.ascii_uppercase) for i in range(12))
		
		self.opensockets  = opensockets
		
		#The buffers are redirected
		kwargs['stdout'] = subprocess.PIPE
		kwargs['stderr'] = subprocess.PIPE
		
		def demote(uid, gid):
			"""
			The UID and GID of the child process is changed to match those of the user
			who issued the command. Otherwise the operation would be executed as root.
			"""
			os.setgid(gid)
			os.setuid(uid)

		kwargs['shell'] = False
		self.process = subprocess.Popen(preexec_fn=demote(user.pw_uid, user.pw_gid), #function executed before the call
										cwd=directory, # The current working directory is changed
										*args, **kwargs)
		
		#The fileno of the stdout buffer is used to make it non-blocking
		self.fd = self.process.stdout.fileno()
		fl = fcntl.fcntl(self.fd, fcntl.F_GETFL) #The file access mode is returned
		fcntl.fcntl(self.fd, fcntl.F_SETFL, fl | os.O_NONBLOCK) # The flags of the file are modified, appending the non-blocking flag blogin

		#The same for stderr
		self.fd_err = self.process.stderr.fileno()
		fl_err = fcntl.fcntl(self.fd_err, fcntl.F_GETFL)
		fcntl.fcntl(self.fd_err, fcntl.F_SETFL, fl | os.O_NONBLOCK)

		#Creation of the line buffer
		self.line_buffer = LineBuffer()

		#Two handlers are registered, each for one of the output buffers. can_read and can_read_stderr act as the callback for events related to both of them
		self.ioloop.add_handler(self.process.stdout, self.can_read, self.ioloop.READ)
		self.ioloop.add_handler(self.process.stderr, self.can_read_stderr, self.ioloop.READ)
	
	def kill(self):
		if self.process.returncode is None:
			self.process.kill()

	def stop(self):
		if self.process.returncode is None:
			print("Sending terminate signal")
			self.process.terminate()
			self.ioloop.call_later(60, self.kill)

	def can_read(self, fd, events):
		"""
		Processes the stdout event
		:param int fd: The file descriptor of the stdout buffer
		:param int events: The event flags (bitwise OR of the constants IOLoop.READ, IOLoop.WRITE, and IOLoop.ERROR)
		"""
		data = self.process.stdout.read(1024)
		

		if len(data) > 0:
			"""If the length of the data is larger than zero, the information is sent to all
			the listening sockets"""
			self.on_data(data, "stdout")

		else:
			print("Lost connection to subprocess")
			self.ioloop.remove_handler(self.process.stdout)
			self.stop_output("stdout")
	
	def can_read_stderr(self, fd, events):
		"""
		Processes the stderr event
		:param int fd: The file descriptor of the stderr buffer
		:param int events: The event flags (bitwise OR of the constants IOLoop.READ, IOLoop.WRITE, and IOLoop.ERROR)
		"""
		data = self.process.stderr.read(1024)

		if len(data) > 0:
			self.on_data(data, "stderr")

		else:
			print("Lost connection to subprocess")
			self.ioloop.remove_handler(self.process.stderr)
			self.stop_output("stderr")

	def on_data(self, data, stream_name):
		"""
		Decodes the data and passes it to on_line
		:param bytes data: an array of bytes with the message
		:param str stream_name: The name of the stream
		"""
		for line in self.line_buffer.read_lines(data):
			self.on_line(line.decode('utf-8'), stream_name)

	def on_line(self, line, stream_name):
		"""
		Sends the line to the open websocket
		:param str line: The message line
		:param str stream_name: The name of the stream
		"""
		for ws in self.opensockets[self.user.pw_name]:
			ws.on_line(self.user.pw_name, self.command, line, self.ip, self.identifier, False, stream_name, shell=self.shell)


	def stop_output(self, stream_name="unknown"):
		"""
		Sends a special message to close the websocket connection
		"""
		for ws in self.opensockets[self.user.pw_name]:
			ws.on_line(self.user.pw_name, self.command, None, self.ip, self.identifier, True, stream_name, shell=self.shell)




# -*- encoding: utf-8 -*-
from __future__ import division
from __future__ import absolute_import
import subprocess
from time import strftime, localtime

def get_data():
    """
    Returns the status information from several system calls/Python modules as a dictionary
    The following keys are returned in the dictionary:

    - time
    - hostname
    - ip
    - uptime
    - kernel_name
    - top
    - memtotal
    - memfree
    - membuffered
    - memcached
    - swaptotal
    - swapfree
    - temperature
    - top_list
    - cpus (itself a list)
    """
    response_dict = {}
    response_dict["time"] = strftime("%a, %d %b %Y %H:%M:%S", localtime())

    response_dict["hostname"] = subprocess.Popen("hostname", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["ip"] = subprocess.Popen("/sbin/ifconfig eth0| grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    if len(response_dict["ip"]) == 0:
        response_dict["ip"] = subprocess.Popen(" /sbin/ifconfig eth0 | grep 'inet\ ' | cut -d: -f2 | awk '{ print $2 }'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')

    response_dict["kernel_name"] = subprocess.Popen("uname -r", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    
    response_dict["memtotal"] = subprocess.Popen("egrep --color 'MemTotal' /proc/meminfo | egrep '[0-9.]{4,}' -o",
                    shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')

    response_dict["memfree"] = subprocess.Popen("egrep --color 'MemFree' /proc/meminfo | egrep '[0-9.]{4,}' -o", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["membuffered"] = subprocess.Popen("egrep --color 'Buffers' /proc/meminfo | egrep '[0-9.]{4,}' -o", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["memcached"] = subprocess.Popen("egrep --color 'Cached' /proc/meminfo | egrep '[0-9.]{4,}' -o", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')

    response_dict["swaptotal"] = subprocess.Popen("egrep --color 'SwapTotal' /proc/meminfo | egrep '[0-9.]{4,}' -o", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["swapfree"] = subprocess.Popen("egrep --color 'SwapFree' /proc/meminfo | egrep '[0-9.]{4,}' -o", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')

    response_dict["temperature"] = float(subprocess.Popen("cat /sys/class/thermal/thermal_zone0/temp", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')) / 1000.0
    response_dict["top"] = subprocess.Popen("top -d 0.5 -b -n2 | grep 'Cpu(s)'|tail -n 1 | awk '{print $2 + $4}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    # response_dict["top"] = subprocess.Popen("top -d 0.5 -b -n2 | tail -n 10 | awk '{print $12}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    # response_dict["uptime"] = subprocess.Popen("uptime | tail -n 1 | awk '{print $1}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["uptime"] = subprocess.Popen("uptime | tail -n 1 | awk '{print $3 $4 $5}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["top_list"] = subprocess.Popen("ps aux --width 30 --sort -rss --no-headers | head  | awk 'BEGIN { OFS = \"-\" } ; {print $3,$4,$11}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')

    response_dict["load_average"] = subprocess.Popen("uptime | awk 'BEGIN { OFS = \"-\" } ; { print $3,$8,$9,$10 }'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["rx"] = subprocess.Popen("ifconfig eth0 | grep \"RX bytes\" | awk '{ print $2 }' | cut -d\":\" -f2", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    response_dict["tx"] = subprocess.Popen("ifconfig eth0 | grep \"TX bytes\" | awk '{ print $6 }' | cut -d\":\" -f2", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8')
    #cpus = subprocess.Popen("mpstat -P ALL | grep -A 5 "+'"%idle"'+ "| tail -n +3 | awk -F"+' " "'+" '{print $ 12 }'",shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8').split('\n')
    cpus = subprocess.Popen("top -d 0.4 -b -n2 | grep \"Cpu\" | tail -n 4 | awk '{print $2 + $4}'", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8').split('\n')

    try:
        cpus.remove("")
    except ValueError:
        pass
    #subprocess.Popen("top -b -n1 | grep Cpu | sed -r 's@.+:\s([0-9\.]+).+@\1@' | awk '{ print $4 }' | grep \"[0-9]\"|cut -f 1 -d '['", shell=True, stdout=subprocess.PIPE).stdout.read().decode('utf-8').split('\n')
    cpus_float = [float(c.replace(',','.')) for c in cpus]
    
    response_dict["cpus"] = cpus_float

    return response_dict





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
import tornado
from tornado.web import Application, RequestHandler, \
StaticFileHandler, asynchronous
from tornado import web, websocket, ioloop
from tornado.httpserver import HTTPServer
from tornado.gen import engine

import os, json, mimetypes

import sys, signal
from os import makedirs
import time

from requests.adapters import HTTPAdapter
from requests_futures.sessions import FuturesSession
from pyjade.ext.tornado import patch_tornado
patch_tornado() #Allows pyjade to work with Tornado

from marcopolo.bindings.marco import Marco, MarcoTimeOutException
from marcopolo.bindings.polo import Polo, PoloInternalException, PoloException

from marcodeployer import utils, conf
import logging


class NotCheckingHostnameHTTPAdapter(HTTPAdapter):
    """
    A middleware that avoids the verification of the SSL Hostname field.
    Since the name of the client cannot be verified,
    it is simply not checked
    
    Generously provided by Juan Luis Boya
    """
    def cert_verify(self, conn, *args, **kwargs):
        """
        Avoids the verification of the SSL Hostname field

        :param Connection conn: The connection object
        """
        super(NotCheckingHostnameHTTPAdapter, self).cert_verify(conn, *args, **kwargs)
        conn.assert_hostname = False

# By changing the adapter no hostname is checked
futures_session = FuturesSession()
futures_session.mount('https://', NotCheckingHostnameHTTPAdapter())

#Creation of the temporal directory if it does not exists
if not os.path.exists(conf.TMPDIR):
    os.makedirs(conf.TMPDIR)

__UPLOADS__ = conf.TMPDIR # temporal directory were files will be stored

open_ws = set() #Set of the current alive websockets

class BaseHandler(RequestHandler):
    """
    The base class which the rest of HTTP handlers extends.
    Provides secure cookie decryption and error handling
    """
    def get_current_user(self):
        """
        Decrypts the secure cookie

        :return: The name of the user or None
        :rtype: string
        """
        return self.get_secure_cookie("user")

    def render(self, template, **kwargs):
        #print(os.path.join(conf.TEMPLATES_DIR, template))
        super(BaseHandler, self).render(os.path.join(conf.TEMPLATES_DIR, template), **kwargs)

    def write_error(self, status_code, **kwargs):
        self.render(os.path.join("500.jade"))


class IndexHandler(BaseHandler):
    """
    In charge of handling GET requests.
    Provides the client with the necessary .html/css/js
    """
    @web.addslash #Appends a '/' at the end of the request
    def get(self):
        """
        Checks if the user is logged and sends the index files
        (basic HTML, CSS and JS).
        If the user is not already logged in, it is redirected 
        to the main page. 
        """
        if not self.current_user:
            self.redirect("/login/")
        else:
            user = tornado.escape.xhtml_escape(self.current_user)
            self.render("index.jade", user=user)

class LoginHandler(BaseHandler):
    """
    Handles login authentication through secure cookies and PAM
    """
    def get(self):
        """
        Returns the login page if the user is not logged.
        Otherwise redirects to the index site.
        """
        if self.current_user:
            self.redirect("/")
        else:
            self.render("login.jade")

    def post(self):
        """
        Processes login requests using PAM. If the user and password combination
        is valid, the response is given a secure cookie, and the user gets 
        redirected to the new index site.
        Otherwise, a 403 page is returned.
        """
        if utils.authenticate(self.get_argument("name"), 
                              self.get_argument("pass")):

            self.set_secure_cookie("user", self.get_argument("name"))
            self.redirect("/")
        else:
            self.set_status(403)
            self.render("badpass.jade")

class Logout(BaseHandler):
    """
    Removes the secure cookie.
    """
    def get(self):
        """
        Removes the secure cookie and redirects the user to the index folder.
        """
        self.clear_cookie("user")
        self.redirect("/")

class UploadAndDeployHandler(BaseHandler):
    """
    Listens for POST requests and performs the deployment asynchronously.
    """
    #The post is asynchronous due to the potencially long deploying time
    @asynchronous
    @engine
    def post(self):
        """
        Receives a set of parameters through an asynchronous POST request:

        - file : A binary stream of data which corresponds to a file uploaded by the user.

        - folder : The folder where the file has to be stored

        - tomcat : Specifies that the service is a Tomcat container and that it must be added to the offered services

        - overwrite : If true, the file will overwrite a previous file with the same name

        - command : A command to execute when after the deployment

        - nodes : The nodes where the file and command are to be deployed

        Writes a status code to the client in return
        """
        file1 = self.request.files['file'][0] #Only one file at a time

        original_fname = file1['filename']
       
        output_file = open(os.path.join(__UPLOADS__, original_fname), 'wb')
        output_file.write(file1['body'])
        output_file.close()
        
        # The nodes where to deploy are returned as a comma-separated string
        nodes = self.get_argument('nodes', '').split(',')[:-1]
        from concurrent import futures
        
        """The deployment process is performed asynchronously 
        using a ThreadPool, which will handle the request asynchronously"""

        futures_set = set()

        for node in nodes:
            future = self.deploy(node=node,
             request=self, 
             filename=original_fname, 
             command=self.get_argument('command', ''), 
             user=self.current_user, 
             folder=self.get_argument('folder', ''), 
             tomcat=self.get_argument('tomcat', ''), 
             overwrite=self.get_argument('overwrite', 'false'))
            
            futures_set.add((future, node))

        error = []
       
        for future, node in futures_set:
            try:
                response = future.result()
                
                if response.status_code > 400:
                    error.append((node, response.reason))
            except Exception as e:
                error.append((node, "Could not connect to the node"))

        if len(error) > 0:
            self.finish("Errors occurred " + 
                " ".join(["Node:"+node+"." for node, reason in error]))
        else:
            self.finish("file" + original_fname + " is uploaded and on deploy")
    
    def deploy(self, node, request, filename, command, user, folder="", idpolo="", tomcat="", overwrite='false'):
        """
        
        Performs the deployment asynchronously.

        
        :param str node: The IP address of the node
        
        :param :class:`BaseHandler` request: The related POST request which invoked this method *Deprecated*
        
        :param str filename: The name of the file to upload
        
        :param str command: The command to execute after deployment
        
        :param str user: The name of the user who performs the request
        
        :param str folder: The deployment folder
        
        :param str idpolo: The id of the polo service to publish
        
        :param str tomcat: Specifies whether the file should be deployed as a tomcat service
        
        :param str overwrite: Specifies if the file can overwrite existing files

        :returns: :class:`concurrent.future` A future that encapsulates the asynchronous execution 
        """
        def get_content_type(filename):
            """
            Guesses the MIME type of the file so it can be sent with the POST request

            :param str filename: The name of the file to process
            """
            return mimetypes.guess_type(filename)[0] or 'application/octet-stream'

        url = "https://"+node+":"+str(conf.RECEIVER_PORT)+"/deploy/"
        
        files = {'file': (filename, 
                    open(os.path.join(__UPLOADS__, filename), 'rb'), 
                    get_content_type(filename))
                }
        
        commands = {'command':command, 
                    'user':user, 
                    'folder': folder, 
                    'idpolo': idpolo, 
                    'tomcat': tomcat,
                    'overwrite':overwrite
                    }
        
        try:
            f = futures_session.post(url, files=files, data=commands, verify=conf.RECEIVERCERT, cert=(conf.APPCERT, conf.APPKEY))
        
            return f
        except Exception as e:
            logging.error("Unknown exception in POSTing %s" % e)

class NodesHandler(websocket.WebSocketHandler):
    """
    Handler for the Polo websocket connection
    """
    def check_origin(self, origin):
        """
        Overrides the parent method to return True for any request, since we are
        working without names. 

        :see: :meth:`check_origin<tornado.websocket.WebSocketHandler.check_origin>`

        :returns: bool True
        """
        return True

    def open(self):
        """
        Processes a new WebSocket connection, storing it in open_ws.
        Returns the nodes offering the deployer service
        """
        open_ws.add(self)
        m = Marco()
        try:
            nodes = m.request_for(conf.RECEIVER_SERVICE_NAME)
            self.write_message(json.dumps({"Nodes":[n.address for n in nodes]}))
        except MarcoTimeOutException:
            self.write_message(json.dumps({"Error": "Error in marco detection"}))
                
    def send_data(self):
        pass

    def on_close(self):
        pass

    def send_update(self):
        pass

class Nodes(RequestHandler):
    """
    Performs a synchronous Marco request for the deployer service
    """
    def get(self):
        """
        Returns a JSON string with the nodes offering the deployer service
        """
        m = Marco()
        try:
            nodes = m.request_for(conf.RECEIVER_SERVICE_NAME)
            self.write(json.dumps({'nodes':[n.address for n in nodes]}))
        except MarcoTimeOutException:
            self.write_message(json.dumps({"Error": "Error in marco detection"}))
        

class ProbeHandler(RequestHandler):
    """
    A test connection to trigger the web browser certificate validation,
    since WebSockets cannot request user confirmation by themselves.
    """
    def get(self):
        self.write("You should be able to open a WebSocket connection now")
        

class ProbeWSHandler(websocket.WebSocketHandler):
    def check_origin(self, origin):
        """
        Overrides the parent method to return True for any request, since we are
        working without names

        :see: :meth:`check_origin<tornado.websocket.WebSocketHandler.check_origin>`

        :returns: bool True
        """
        return True

    def open(self):
        self.write_message("OK")
        self.close()


routes = [
    (r'/', IndexHandler),
    (r'/nodes/?', Nodes),
    (r'/static/(.*)', StaticFileHandler, {"path":conf.STATIC_PATH}),
    (r'/ws/nodes/?', NodesHandler),
    (r"/login/?", LoginHandler),
    (r"/logout/?", Logout),
    (r'/upload/?', UploadAndDeployHandler),
    
    #probes
    (r'/probe/?', ProbeHandler),
    (r'/ws/probe/?', ProbeWSHandler)
]

class RedirectHandler(RequestHandler):
    """
    Redirects all request to the secure port
    """
    def get(self):
        """
        Redirects all requests to the secure port
        """
        self.redirect("https://%s:%s%s" 
            % ((self.request.host).replace(":"+str(conf.NON_SECURE_DEPLOYER_PORT), ""), 
                conf.DEPLOYER_PORT, self.request.uri), permanent=True
            )

nonsecure_routes = [
    (r'/.*', RedirectHandler)    
]

settings = {
    "debug": True,
    "static_path": conf.STATIC_PATH,
    "login_url":"/login/",
    "cookie_secret":"2a70b29a80c23f097a074626e584c8f60a87cf33f518f0eda60db0211c82"
}

app = Application(routes, **settings)
nonsecure_app = Application(nonsecure_routes)
io_loop = ioloop.IOLoop.instance()


def shutdown():
    logging.info("Stopping gracefully")
    try:
        Polo().unpublish_service(conf.DEPLOYER_SERVICE_NAME, delete_file=True)
    except Exception as e:
        logging.warning(e)
    io_loop.stop()

def sigint_handler(signal, frame):
    io_loop.add_callback(shutdown)

signal.signal(signal.SIGINT, sigint_handler)


def main(args=None):
    
    pid = os.getpid()

    logging.basicConfig(filename=conf.DEPLOYER_LOG_FILE, level=getattr(logging, conf.DEPLOYER_LOGLEVEL.upper()))

    #Replace with SSLContext (this option is maintained for compatibility reasons)
    httpServer = HTTPServer(app, ssl_options={ 
        "certfile": conf.APPCERT,
        "keyfile": conf.APPKEY,
    })

    httpServer.listen(conf.DEPLOYER_PORT)

    nonsecure_app.listen(conf.NON_SECURE_DEPLOYER_PORT)
    
    while True:
        try:
            Polo().publish_service(conf.DEPLOYER_SERVICE_NAME, root=True)
            break
        except PoloInternalException as e:
            logging.warning(e)
            time.sleep(1)
        except PoloException as i:
            logging.warning(i)
            break

    logging.info("Serving on port %d" % conf.DEPLOYER_PORT)
    io_loop.start()

if __name__ == "__main__":
    main(sys.argv[1:])


# -*- coding: utf-8 -*-
from os.path import dirname, join, isabs
import tempfile
import logging
import six
from six.moves import configparser

CONF_DIR = '/etc/marcodeployer'

CERTS = "/usr/lib/marcodeployer/certs"
APPCERT = "app.crt"
APPKEY = "app.key"
RECEIVERCERT = "receiver.crt"
RECEIVERKEY = "receiver.key"

TMPDIR = join(tempfile.gettempdir(), "tmp_deployer")

STATIC_PATH = "/usr/lib/marcodeployer/static"
TOMCAT_PATH = '/var/lib/tomcat7/webapps/'

DEPLOYER_PORT = 1342
RECEIVER_PORT = 1339
RECEIVER_WEBSOCKET_PORT = 1370
NON_SECURE_DEPLOYER_PORT = 1442

INTERFACE = 'eth0'

REFRESH_FREQ = 1000.0

STATUS_MONITOR_SERVICE_NAME = "statusmonitor"
DEPLOYER_SERVICE_NAME = "deployer"
RECEIVER_SERVICE_NAME = "receiver"

TEMPLATES_DIR = "/usr/lib/marcodeployer/templates/"

LOGGING_DIR = "/var/log/marcopolo"
DEPLOYER_LOG_FILE = "/var/log/marcodeployer/marcodeployerd.log"
RECEIVER_LOG_FILE = "/var/log/marcodeployer/marcoreceiverd.log"

DEPLOYER_LOGLEVEL = "DEBUG"
RECEIVER_LOGLEVEL = "DEBUG"

SECRET_FILE = "/etc/marcodeployer/secret"


default_values = {
    "CERTS": CERTS,
    "APPCERT": APPCERT if isabs(APPCERT) else join(CERTS, APPCERT),
    "APPKEY": APPKEY if isabs(APPKEY) else join(CERTS, APPKEY),
    "RECEIVERCERT": RECEIVERCERT if isabs(RECEIVERCERT) else join(CERTS, RECEIVERCERT),
    "RECEIVERKEY": RECEIVERKEY if isabs(RECEIVERKEY) else join(CERTS, RECEIVERKEY),
    "TMPDIR" : TMPDIR if isabs(TMPDIR) else join(tempfile.gettempdir(), TMPDIR),
    "STATIC_PATH": STATIC_PATH,
    "TOMCAT_PATH": TOMCAT_PATH,
    "DEPLOYER_PORT" : DEPLOYER_PORT,
    "RECEIVER_PORT" : RECEIVER_PORT,
    "RECEIVER_WEBSOCKET_PORT" : RECEIVER_WEBSOCKET_PORT,
    "INTERFACE":INTERFACE,
    "REFRESH_FREQ": REFRESH_FREQ,
    "STATUS_MONITOR_SERVICE_NAME" : STATUS_MONITOR_SERVICE_NAME,
    "DEPLOYER_SERVICE_NAME" : DEPLOYER_SERVICE_NAME,
    "RECEIVER_SERVICE_NAME" : RECEIVER_SERVICE_NAME,
    "TEMPLATES_DIR" : TEMPLATES_DIR,
    "LOGGING_DIR" : LOGGING_DIR,
    "DEPLOYER_LOG_FILE" : DEPLOYER_LOG_FILE,
    "RECEIVER_LOG_FILE" : RECEIVER_LOG_FILE,
    "DEPLOYER_LOGLEVEL" : DEPLOYER_LOGLEVEL,
    "RECEIVER_LOGLEVEL" : RECEIVER_LOGLEVEL,
    "SECRET_FILE": SECRET_FILE,
    "NON_SECURE_DEPLOYER_PORT" : NON_SECURE_DEPLOYER_PORT,
}

config = configparser.RawConfigParser(default_values, allow_no_value=False)

DEPLOYER_FILE_READ = join(CONF_DIR, 'deployer.cfg')


try:
    with open(DEPLOYER_FILE_READ, 'r') as df:
        config.readfp(df)

        CERTS = config.get('common', 'CERTS')
        
        APPCERT = config.get('deployer', 'APPCERT')
        APPCERT = APPCERT if isabs(APPCERT) else join(CERTS, APPCERT)

        APPKEY = config.get('deployer', 'APPKEY')
        APPKEY = APPKEY if isabs(APPKEY) else join(CERTS, APPKEY)

        RECEIVERCERT = config.get('receiver', 'RECEIVERCERT')
        RECEIVERCERT = RECEIVERCERT if isabs(RECEIVERCERT) else join(CERTS, RECEIVERCERT)
        
        RECEIVERKEY = config.get('receiver', 'RECEIVERKEY')
        RECEIVERKEY = RECEIVERKEY if isabs(RECEIVERKEY) else join(CERTS, RECEIVERKEY)

        TMPDIR = config.get('deployer', 'TMPDIR')
        
        STATIC_PATH = config.get('deployer', 'STATIC_PATH')
        TOMCAT_PATH = config.get('deployer', 'TOMCAT_PATH')
        DEPLOYER_PORT = config.getint('deployer', 'DEPLOYER_PORT')
        RECEIVER_PORT = config.getint('receiver', 'RECEIVER_PORT')
        RECEIVER_WEBSOCKET_PORT = config.getint('receiver', 'RECEIVER_WEBSOCKET_PORT')
        INTERFACE = config.get('receiver', 'INTERFACE')
        REFRESH_FREQ = config.getfloat('receiver', 'REFRESH_FREQ')
        STATUS_MONITOR_SERVICE_NAME = config.get('receiver', 'STATUS_MONITOR_SERVICE_NAME')
        DEPLOYER_SERVICE_NAME = config.get('deployer', 'DEPLOYER_SERVICE_NAME')
        RECEIVER_SERVICE_NAME = config.get('receiver', 'RECEIVER_SERVICE_NAME')
        TEMPLATES_DIR = config.get('deployer', 'TEMPLATES_DIR')
        LOGGING_DIR = config.get('common', 'LOGGING_DIR')
        DEPLOYER_LOG_FILE = config.get('deployer', 'DEPLOYER_LOG_FILE')
        RECEIVER_LOG_FILE = config.get('receiver', 'RECEIVER_LOG_FILE')
        DEPLOYER_LOGLEVEL = config.get('deployer', 'DEPLOYER_LOGLEVEL')
        RECEIVER_LOGLEVEL = config.get('receiver', 'RECEIVER_LOGLEVEL')
        SECRET_FILE = config.get('receiver', 'SECRET_FILE')
        NON_SECURE_DEPLOYER_PORT = config.get('deployer', 'NON_SECURE_DEPLOYER_PORT')

except IOError as i:
    logging.warning("Warning! The configuration file could not be read. Defaults will be used as fallback")
except Exception as e:
    logging.warning("Unknown exception in configuration parser %s" % e)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from os import path
from crypt import crypt
from re import compile as compile_regex

import pam
import netifaces as ni
from netifaces import AF_INET
import six
import spwd
import pwd
#http://code.activestate.com/recipes/578489-system-authentication-against-etcshadow-or-etcpass/
def authenticate(name, password):
    """
    Returns true or false depending on the success of the name-password combination using
    the shadows or passwd file (The shadow file is preferred if it exists) 
    """
    try:
        success = pam.pam().authenticate(name, password)
        if success is True:
            return success
    except Exception as e:
        logging.warning(e)
        return False
        
    if path.exists("/etc/shadow"):
        
        try:
            if six.PY3:
                shadow = spwd.getspnam(name).sp_pwdp # https://docs.python.org/3.4/library/spwd.html#module-spwd
            else:
                shadow = spwd.getspnam(name).sp_pwd
        except KeyError as e:
            return False
    else:
        shadow = pwd.getpwnam(name).pw_passwd
    
    salt_pattern = compile_regex(r"\$.*\$.*\$")
    
    try:
        salt = salt_pattern.match(shadow).group()
    except AttributeError as a:
        logging.warning(a)
        return False
    return crypt(password, salt) == shadow


def getip(interface, protocol=None, host=None):
    """
    Returns the IP associated with the configured interface
    """
    return ni.ifaddresses(interface).get(AF_INET)[0].get('addr')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from tornado.web import Application, RequestHandler, StaticFileHandler
from tornado import ioloop
from tornado.websocket import WebSocketHandler
import tornado.web
from tornado.httpserver import HTTPServer 
from tornado import ioloop
from tornado.gen import coroutine

import os, time
from shlex import split
from subprocess import Popen
import subprocess
import pwd
import sys


sys.path.append(os.path.realpath(__file__))


from tornado.ioloop import PeriodicCallback
import signal
from os import path, makedirs

import subprocess
import fcntl
import sys
import json

import socket
import string, random
import hashlib
from concurrent import futures
from tornado.web import decode_signed_value
import logging
import ssl

from marcopolo.bindings.polo import Polo, PoloInternalException, PoloException

import six
from six.moves.urllib import parse as urlparse

from marcodeployer.bufferprocessor import ProcessReactor
from marcodeployer.statusmonitor import get_data
from marcodeployer.utils import getip 
from marcodeployer import conf

#ip = ""
ip = getip(conf.INTERFACE)
opensockets={}
io_loop = ioloop.IOLoop.instance()
data_dict = {}
data_json = ""

response_dict = {}
statusmonitor_open_sockets =  []
getDataCallback = None
processes = {}

def shutdown():
    """
    Stops the application gracefully, closing all socket connections
    and unpublishing the MarcoPolo services.
    """
    logging.info("Stopping gracefully")
    for socket in statusmonitor_open_sockets:
        socket.close()
    try:
        Polo().unpublish_service(conf.RECEIVER_SERVICE_NAME, delete_file=True)
        Polo().unpublish_service(conf.STATUS_MONITOR_SERVICE_NAME, delete_file=True)
    except Exception as e:
        logging.warning(e)
    io_loop.stop()

def sigint_handler(signal, frame):
    io_loop.add_callback(shutdown)

signal.signal(signal.SIGINT, sigint_handler)
signal.signal(signal.SIGTERM, sigint_handler)

class DeployHandler(RequestHandler):
    @tornado.web.asynchronous
    def post(self):
        """
        POST handler received from ``deployer.py``. 
        It handles the deployment of the file and the execution of the desired command.
        """

        file1 = self.request.files['file'][0]
        
        command = self.get_argument('command', '')
        idpolo = self.get_argument('idpolo', '')
        tomcat = self.get_argument('tomcat', '')
        
        if not tomcat:
            folder = self.get_argument('folder', '')
        else:
            folder = conf.TOMCAT_PATH

        fname = file1['filename']
        
        user = self.get_argument('user', '')
        
        user_pwd = pwd.getpwnam(user)
        
        #Handling of relative paths
        folder = folder.replace('~', user_pwd.pw_dir, 1)
        
        if len(folder) == 0 or folder[0] != '/':
            folder = os.path.join(user_pwd.pw_dir, folder)

        if folder == '':
            folder = user_pwd.pw_dir

        if not os.path.isdir(folder):
            return

        if not os.path.exists(folder):
            makedirs(folder)
            chown(folder, user.pw_uid, user.pw_gid)


        final_directory = os.path.join(folder, fname)
        
        overwrite = self.get_argument('overwrite', 'false')
        
        overwrite = False if overwrite.lower() == 'false' else True;


        thread_pool = futures.ThreadPoolExecutor(max_workers=1)
        thread_pool.submit(self.execute, command=command, file_desc=file1, filename=final_directory, directory=folder, user=user_pwd, tomcat=tomcat, overwrite=overwrite)
        
        self.finish('OK')

    @tornado.web.asynchronous
    def execute(self, command, file_desc, filename, directory, user, tomcat=False, overwrite="false"):
        
        if os.path.isfile(filename) and not overwrite:
            return

        def demote(user_uid, user_gid):
            os.setgid(user_gid)
            os.setuid(user_uid)
            
        if os.path.exists(os.path.dirname(filename)):
            output_file = open(filename, 'wb')
        else:
            return

        output_file.write(file_desc['body'])
        if not tomcat:
            os.chown(filename, user.pw_uid, user.pw_gid)
        else:
            os.chown(filename, pwd.getpwnam('tomcat7').pw_uid, pwd.getpwnam('tomcat7').pw_gid)
        output_file.close()
        if len(command) > 0:
            p = ProcessReactor(user, directory, io_loop, ip, opensockets, split(command), shell=False)
            if processes.get(user.pw_name, None) is None:
                processes[user.pw_name] = set()
            processes[user.pw_name].add(p)

secret = ""
try:
    with open(conf.SECRET_FILE, 'r') as secret_file:
        secret = secret_file.read()
except Exception as e:
    logging.error("Could not open secret file")
    #sys.exit(1)

settings = {
    "debug": True,
    "static_path": conf.STATIC_PATH, #os.path.join(os.path.dirname(__file__), "static"),
    "cookie_secret": secret
}

class LoggerHandler(WebSocketHandler):
    """
    Processes the logging messages
    """
    def check_origin(self, origin):
        """
        Overrides the parent method to return True for any request, since we are
        working without names

        :returns: bool True
        """
        return True
    
    def open(self):
        """
        Notifies the opening of a new Logger connection
        """
        logging.debug("A new connection was open")

    def on_message(self, message):
        """
        A message is sent by the client after creating the connection. The method verifies the user
        secret cookie and appends the connection to the opensockets dictionary.

        :param str message: The received message
        """
        user_id = decode_signed_value(settings["cookie_secret"], 'user', json.loads(message).get("register", "")).decode('utf-8')
        
        """
        If the user_id is other than None the verification has succeded, and the connection is appended to the 
        rest of the websockets related to the user.
        """
        if user_id is not None:
            if opensockets.get(user_id) is None:
                opensockets[user_id] = []
            opensockets[user_id].append(self)
        else:
            logging.debug("The user could not be found")

    def on_line(self, user, command, message, ip, identifier, stop=False, stream_name="stdout", *args, **kwargs):
        """
        The io_loop calls the function when a new message appears.
        :param str user: The name of the user
        :param str command: The command in execution
        :param str message: The message to deliver
        :param str ip: The ip of the server, so the client knows where the message comes from
        :param boolean stop: Determines if the connection must be closed or not *deprecated*
        :param str stream_name: The name of the stream
        """
        
        msg = {}
        msg["user"] = user
        msg["command"] = command
        msg["message"] = message
        msg["ip"] = ip
        msg["identifier"] = identifier
        msg["stop"] = stop
        msg["stream_name"] = stream_name
        msg["shell"] = kwargs.get("shell", False)
        self.write_message(json.dumps(msg))

    def on_close(self):
        """
        Removes the connection from the opensockets dictionary
        """
        
        for ws in opensockets:
            if self in opensockets[ws]:
                opensockets[ws].remove(self)
                
                break
        else:
            logging.debug("The websocket could not be found")

class ShellHandler(LoggerHandler):
    
    def on_message(self, message):
        message_json = message#.decode('utf-8')

        try:
            message_dict = json.loads(message_json)
        except ValueError as v:
            return
        
        if message_dict.get("register", None) is not None:
            user_id = decode_signed_value(settings["cookie_secret"], 'user', message_dict["register"]).decode('utf-8')
            
            if not user_id is None:
                if opensockets.get(user_id) is None:
                    opensockets[user_id] = []
                opensockets[user_id].append(self)
            else:
                pass

        elif message_dict.get("command", None) is not None:
            user_id = decode_signed_value(settings["cookie_secret"], 'user', message_dict.get("user_id", ""))
            
            if user_id is not None:
                user_id = user_id.decode('utf-8')
                user_pwd = pwd.getpwnam(user_id)
                try:
                    command = message_dict["command"]
                    p = ProcessReactor(user_pwd, user_pwd.pw_dir, io_loop, ip, opensockets, split(command), shell=True)
                    if processes.get(user_id, None) is None:
                        processes[user_id] = set()
                    processes[user_id].add(p)
                except Exception as e:
                    logging.warning(e)

        elif message_dict.get("remove", None) is not None:
            logging.debug("remove")
            user_id = decode_signed_value(settings["cookie_secret"], 'user', message_dict.get("user_id", ""))
            if user_id is not None:
                identifier = message_dict.get("remove", None)
                if identifier is not None:
                    print("identifier", identifier)
                    process = next((x for x in processes.get(user_id, set()) if x.identifier == identifier), None)
                    if process is not None:
                        process.stop()
                        processes[user_id].remove(process)

        elif message_dict.get("removeshell", None) is not None:
            logging.debug("removeshell")
            user_id = decode_signed_value(settings["cookie_secret"], 'user', message_dict.get("user_id", ""))
            if user_id is not None:
                identifiers = message_dict.get("removeshell", None)
                if identifiers is not None:
                    try:
                        identifiers_dict = message_dict["removeshell"]
                        for identifier in identifiers:
                            logging.debug(identifier)
                            logging.debug(processes.get(user_id, set()))
                            process = next((x for x in processes.get(user_id, set()) if x.identifier == identifier), None)
                            if process is not None:
                                logging.debug("Killing")
                                process.stop()
                                processes[user_id].remove(process)
                    except ValueError as v:
                        logging.warning(v)
                    except KeyError as k:
                        logging.warning(k)


class ProbeWSHandler(WebSocketHandler):
    def check_origin(self, origin):
        """
        Overrides the parent method to return True for any request, since we are
        working without names

        :returns: bool True
        """
        return True

    def open(self):
        """
        Returns a confirmation message
        """
        self.write_message("OK")
        self.close()


class ProbeHandler(RequestHandler):
    def get(self):
        self.write("You should be able to create websocket connections now")


def start_callback():
    """
    Starts the collection callback

    """
    global getDataCallback
    if getDataCallback is None:
        getDataCallback = PeriodicCallback(process_data, conf.REFRESH_FREQ)  
        getDataCallback.start()
    elif not getDataCallback.is_running():
        getDataCallback.start()

def stop_callback():
    """
    If the number of open connections is zero, stops the collection callback of data.
    """
    global getDataCallback
    if getDataCallback is not None:
        if len(statusmonitor_open_sockets) == 0:
            getDataCallback.stop()

def process_data():
    """
    Processes the statusmonitor data
    """
    global data_dict, data_json
    if len(statusmonitor_open_sockets) > 0:
        data_dict = get_data()
        data_json = json.dumps(data_dict,separators=(',',':'))


class SocketHandler(WebSocketHandler):

    def check_origin(self, origin):
        """
        Overrides the parent method to return True for any request, since we are
        working without names

        :returns: bool True
        """
        return True

    def open(self):
        logging.info("Connection open from " + self.request.remote_ip)
        if not self in statusmonitor_open_sockets:
            statusmonitor_open_sockets.append(self) #http://stackoverflow.com/a/19571205
        self.callback = PeriodicCallback(self.send_data, 1000)

        self.callback.start()
        start_callback()

    def send_data(self):
        self.write_message(data_json)
        return

        
    def on_close(self):
        self.callback.stop()
        if self in statusmonitor_open_sockets:
            statusmonitor_open_sockets.remove(self)

        stop_callback()

    def send_update(self):
        pass



routes =  [
    (r'/deploy/?', DeployHandler),
]

routes_ws = [
    (r'/ws/probe/', ProbeWSHandler),
    (r'/ws/status/', SocketHandler),
    (r'/ws/logger/', ShellHandler),
    (r'/probe/', ProbeHandler),
    (r'/', ProbeHandler),
]

app = Application(routes, **settings)

wsapp = Application(routes_ws, **settings);


def main(args=None):
    """
    Creates the servers, initializes the logging facility, publishes the ``MarcoPolo`` services and starts the ``io_loop``.
    """
    ip = getip(conf.INTERFACE)
    pid = os.getpid()

    #if not os.path.exists('/var/run/marcopolo'):
    #    makedirs('/var/run/marcopolo')
    logging.basicConfig(filename=conf.RECEIVER_LOG_FILE, level=getattr(logging, conf.RECEIVER_LOGLEVEL.upper()))


    httpServer = HTTPServer(app, ssl_options={
        "certfile": conf.RECEIVERCERT,
        "keyfile": conf.RECEIVERKEY,
        "cert_reqs": ssl.CERT_REQUIRED,
        "ca_certs": conf.APPCERT,
    })

    httpServer.listen(conf.RECEIVER_PORT)

    wsapp.listen(conf.RECEIVER_WEBSOCKET_PORT, 
            ssl_options={"certfile": conf.APPCERT,
                         "keyfile": conf.APPKEY})


    while True:
        try:
            Polo().publish_service(conf.RECEIVER_SERVICE_NAME, root=True)
            Polo().publish_service(conf.STATUS_MONITOR_SERVICE_NAME, root=True)
            break
        except PoloInternalException as e:
            logging.warning(e)
            time.sleep(1)
        except PoloException as i:
            logging.warning(i)
            break
    logging.info("Starting receiver on port %d. WebSockets on %d" % (conf.RECEIVER_PORT, conf.RECEIVER_WEBSOCKET_PORT))
    io_loop.start()


if __name__ == "__main__":
    main(sys.argv[1:])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A deployer, remote shell and status monitor built upon MarcoPolo
"""

from setuptools import setup, find_packages
from distutils.command.clean import clean
from distutils.command.install import install
from codecs import open
import os, sys, subprocess
import glob
import stat

custom_deployer_params = [
                            "--marcodeployer-disable-daemons",
                            "--marcodeployer-disable-deployer",
                            "--marcodeployer-enable-receiver",
                            "--marcodeployer-no-start"
                         ]

def detect_init():
    try:
        subprocess.check_call(["systemctl", "--version"], stdout=None, stderr=None, shell=False)
        return 0
    except (subprocess.CalledProcessError, OSError):
        return 1

def enable_service(service):
    sys.stdout.write("Enabling service " + service +"...\n")
    if init_bin == 0:
        subprocess.call(["systemctl", "enable", service], shell=False)
    else:
        subprocess.call(["update-rc.d", "-f", service, "remove"], shell=False)
        subprocess.call(["update-rc.d", service, "defaults"], shell=False)
    
    sys.stdout.write("Enabled!\n")

def start_service(service):
    sys.stdout.write("Starting service " + service + "...")
    if init_bin == 0:
        subprocess.call(["systemctl", "stop", service], shell=False)
        subprocess.call(["systemctl", "start", service], shell=False)
    else: 
        subprocess.call(["service", service, "stop"], shell=False)
        subprocess.call(["service", service, "start"], shell=False)

    sys.stdout.write("Started!\n")

if __name__ == "__main__":

    deployer_params = []

    python_version = int(sys.version[0])

    for param in sys.argv:
        if param in custom_deployer_params:
            deployer_params.append(param)
            sys.argv.remove(param)

    here = os.path.abspath(os.path.dirname(__file__))

    long_description = ""
    
    with open(os.path.join(here, "DESCRIPTION.rst"), encoding='utf-8') as f:
        long_description = f.read()

    def copy_dir(directory, module):
        dir_path = directory
        base_dir = os.path.join(module, dir_path)
        for (dirpath, dirnames, files) in os.walk(base_dir):
            for f in files:
                yield os.path.join(dirpath.split('/', 1)[1], f)

    data_files = [
                    ('/usr/lib/marcodeployer/static/css', glob.glob("usr/lib/marcodeployer/static/css/*")),
                    ('/usr/lib/marcodeployer/static/fonts', glob.glob("usr/lib/marcodeployer/static/fonts/*")),
                    ('/usr/lib/marcodeployer/static/img',  glob.glob("usr/lib/marcodeployer/static/img/*")),
                    ('/usr/lib/marcodeployer/static/js',  glob.glob("usr/lib/marcodeployer/static/js/*")),
                    ('/usr/lib/marcodeployer/certs', glob.glob("usr/lib/marcodeployer/certs/*")),
                    ('/usr/lib/marcodeployer/templates', glob.glob("usr/lib/marcodeployer/templates/*.jade")),
                    ('/etc/marcodeployer', glob.glob('etc/marcodeployer/*'))
                 ]
    
    daemon_files = []
    if "--marcodeployer-disable-daemons" not in deployer_params:
        init_bin = detect_init()

        if init_bin == 1:
            daemon_files = [
                            ('/etc/init.d/', ["daemon/systemv/marcodeployerd", 
                                              "daemon/systemv/marcoreceiverd"])
                            ]

        else:
            daemon_files = [
                            ('/etc/systemd/system/', ["daemon/systemd/marcodeployerd.service", 
                                                      "daemon/systemd/marcoreceiverd.service"])
                           ]

    data_files.extend(daemon_files)

    description = "The deployer for the marcopolo environment"

    setup(
        name="marcopolo-deployer",
        provides=["marcodeployer"],
        version='0.1.6',
        description=description,
        long_description = long_description,
        url="marcopolo.martinarroyo.net/apps/marcodeployer",
        author="Diego Martín",
        author_email="martinarroyo@usal.es",
        license="MIT",
        classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',

        'Topic :: Software Development',
        'Topic :: System :: Networking',
        'Topic :: System :: Monitoring',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Natural Language :: English',

        ],
        keywords="marcopolo deployer statusmonitor",
        packages=find_packages(),
        install_requires=[
            'certifi>=2015.4.28',
            'netifaces>=0.10.4',
            'pyjade>=3.0.0',
            'python-pam>=1.8.2',
            'requests>=2.7.0',
            'requests-futures>=0.9.5',
            'six>=1.9.0',
            'tornado>=4.1',
            'marcopolo>=0.1.0',
            'marcopolo.bindings>=0.1.0',
        #    'transifex-client',
        #    'sphinx-intl'
        ],
        zip_safe=False,
        data_files=data_files,
        entry_points={
            'console_scripts':[
                'marcodeployerd = marcodeployer.deployer:main',
                'marcoreceiverd = marcodeployer.receiver:main'
            ]
        }
    )

    if "install" in sys.argv:
        if "--marcodeployer-disable-daemons" not in deployer_params:
            if "--marcodeployer-disable-deployer" not in deployer_params:
                enable_service("marcodeployerd")
                if "--marcodeployer-no-start" not in deployer_params:
                    start_service("marcodeployerd")

            if "--marcodeployer-enable-receiver" in deployer_params:
                enable_service("marcoreceiverd")
                if "--marcodeployer-no-start" not in deployer_params:
                    start_service("marcoreceiverd")

        if not os.path.exists("/var/log/marcodeployer"):
            os.makedirs('/var/log/marcodeployer')

        for f in os.listdir('/usr/lib/marcodeployer/certs'):
            os.chmod(os.path.join('/usr/lib/marcodeployer/certs', f), stat.S_IREAD | stat.S_IWRITE)

        os.chmod('/etc/marcodeployer/secret', stat.S_IREAD | stat.S_IWRITE)